-- MIT License
--
-- Copyright (c) 2012 Brendan Zagaeski
--
-- Permission is hereby granted, free of charge, to any person obtaining a copy
-- of this software and associated documentation files (the "Software"), to deal
-- in the Software without restriction, including without limitation the rights
-- to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is
-- furnished to do so, subject to the following conditions:
--
-- The above copyright notice and this permission notice shall be included in all
-- copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-- OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-- SOFTWARE.

Avatar = {}
Map = {}
Box = {
  colors = { {200, 200, 0}, {200, 0, 200}, {100, 240, 100}, {0, 200, 200}, {240, 100, 100}, { 100, 100, 240 }},
  size = 22
}

function Avatar:load()
  self.width = 15
  self.height = 15
  self.x = love.graphics.getWidth()/2 - Avatar.width/2
  self.y = love.graphics.getHeight()/2 - Avatar.width/2
end

function Map:load()
  self.x = 0
  self.y = 0
  self.eaten = {}
  self.spacing = 40
  self.width = love.graphics.getWidth()
  self.height = love.graphics.getHeight()
  self.maxspeed = 2
  self.xvelocity = 0
  self.yvelocity = 0
  self.accel = 0.5
  self.drag = 0.95
end

function Avatar:draw()
  love.graphics.push()
  love.graphics.setColor(220, 20, 20, 255)
  love.graphics.rectangle("fill", self.x, self.y, self.width, self.height)
  love.graphics.pop()
end

function Map:move()
  if love.keyboard.isDown("right") then
    self.xvelocity = self.xvelocity + self.accel
  elseif love.keyboard.isDown("left") then
    self.xvelocity = self.xvelocity - self.accel
  end

  if love.keyboard.isDown("up") then
    self.yvelocity = self.yvelocity - self.accel
  elseif love.keyboard.isDown("down") then
    self.yvelocity = self.yvelocity + self.accel
  end

  if (math.abs(self.yvelocity) < 0.5) then
    self.yvelocity = 0
  else
    self.yvelocity = self.yvelocity / math.abs(self.yvelocity) * math.min(math.abs(self.yvelocity), self.maxspeed)
  end
  if (math.abs(self.xvelocity) < 0.5) then
    self.xvelocity = 0
  else
    self.xvelocity = self.xvelocity / math.abs(self.xvelocity) * math.min(math.abs(self.xvelocity), self.maxspeed)
  end

  self.x = self.x + self.xvelocity
  self.y = self.y + self.yvelocity

  self.xvelocity = self.xvelocity * self.drag
  self.yvelocity = self.yvelocity * self.drag
end

function Map:gridsnap(x)
  local y = math.floor(x * 2 / self.spacing)
  return (y + math.abs(y % 2)) / 2 * self.spacing
end

function Map:eat()
  local center = { x = (self.x + self.width / 2), y = (self.y + self.height / 2) }
  local nearestbox = { x = Map:gridsnap(center.x), y = Map:gridsnap(center.y) }
  if math.abs(center.x - nearestbox.x) < (Box.size / 2 + Avatar.width / 2)  and math.abs(center.y - nearestbox.y) < (Box.size / 2 + Avatar.height / 2) then
    if not self.eaten then
      self.eaten = {}
    end
    if not self.eaten[nearestbox.x] then
      self.eaten[nearestbox.x] = {}
    end
    self.eaten[nearestbox.x][nearestbox.y] = 1
  end
end

function Map:update()
  Map:move()
  Map:eat()
end

function Map:draw()
  love.graphics.push()
  love.graphics.translate(-self.x, -self.y)

  local startx = Map:gridsnap(self.x)
  local starty = Map:gridsnap(self.y)

  for i=startx, startx + self.width, self.spacing do
    for j=starty, starty + self.height, self.spacing do
      if self.eaten and self.eaten[i] then
        if not self.eaten[i][j] then
          Box:draw(i, j)
        end
      else
        Box:draw(i, j)
      end
    end
  end
  love.graphics.pop()
end

function Box:draw(x, y)
  love.graphics.push()
  local corner = { x = x - self.size / 2, y = y - self.size / 2 }
  love.graphics.setColor(Box.colors[math.floor(math.abs(x) / Map.spacing  + math.abs(y) / Map.spacing) % 6 + 1])
  love.graphics.rectangle("fill", corner.x, corner.y, self.size, self.size)
  --love.graphics.print(x .. "," .. y, corner.x, corner.y + Box.size + 10)
  love.graphics.pop()
end

function love.load()
  --love.graphics.setMode(640, 480, false, false, 0)
  Avatar:load()
  Map:load()
end

function love.update()
  Map:update()
end

function love.draw()
  Map:draw()
  Avatar:draw()
end
